"""
    Status messages tests.
"""

import unittest
from unittest import TestSuite
from zope.testing.doctestunit import DocTestSuite

def test_suite():
    return TestSuite((
        DocTestSuite('Products.statusmessages.adapter'),
        DocTestSuite('Products.statusmessages.message'),
        ))

if __name__ == '__main__':
    unittest.main(defaultTest="test_suite")



import struct

from zope.interface import implements

from Products.statusmessages.interfaces import IMessage


def _utf8(value):
    if isinstance(value, unicode):
        return value.encode('utf-8')
    elif isinstance(value, str):
        return value
    return ''

def _unicode(value):
    return unicode(value, 'utf-8', 'ignore')


class Message:
    """A single status message.

    Let's make sure that this implementation actually fulfills the
    'IMessage' API.

      >>> from zope.interface.verify import verifyClass
      >>> verifyClass(IMessage, Message)
      True
    
      >>> status = Message(u'this is a test', type=u'info')
      >>> status.message
      u'this is a test'

      >>> status.type
      u'info'

    It is quite common to use MessageID's as status messages:

      >>> from zope.i18nmessageid import MessageFactory
      >>> from zope.i18nmessageid import Message as I18NMessage
      >>> msg_factory = MessageFactory('test')

      >>> msg = msg_factory(u'test_message', default=u'Default text')

      >>> status = Message(msg, type=u'warn')
      >>> status.type
      u'warn'

      >>> type(status.message) is I18NMessage
      True

      >>> status.message.default
      u'Default text'

      >>> status.message.domain
      'test'

    """
    implements(IMessage)

    def __init__(self, message, type=''):
        self.message = message
        self.type = type

    def __eq__(self, other):
        if not isinstance(other, Message):
            return False
        if self.message == other.message and self.type == other.type:
            return True
        return False

    def encode(self):
        """
        Encode to a cookie friendly format.
        
        The format consists of a two bytes length header of 11 bits for the
        message length and 5 bits for the type length followed by two values.
        """
        message = _utf8(self.message)[:0x3FF] # we can store 2^11 bytes
        type = _utf8(self.type)[:0x1F]        # we can store 2^5 bytes
        size = (len(message) << 5) + (len(type) & 31) # pack into 16 bits
        
        return struct.pack('!H%ds%ds' % (len(message), len(type)), 
                           size, message, type)

def decode(value):
    """
    Decode messages from a cookie

    We return the decoded message object, and the remainder of the cookie
    value (it can contain further messages).

    We expect at least 2 bytes (size information).
    """
    if len(value) >= 2:
        size = struct.unpack('!H', value[:2])[0]
        msize, tsize = (size >> 5, size & 31)
        message = Message(_unicode(value[2:msize+2]),
                          _unicode(value[msize+2:msize+tsize+2]))
        return message, value[msize+tsize+2:]
    return None, ''



from base64 import encodestring, decodestring
import binascii
import sys

from zope.annotation.interfaces import IAnnotations
from zope.i18n import translate
from zope.interface import implements

from Products.statusmessages import STATUSMESSAGEKEY
from Products.statusmessages.message import decode
from Products.statusmessages.message import Message
from Products.statusmessages.interfaces import IStatusMessage

import logging
logger = logging.getLogger('statusmessages')

class StatusMessage(object):
    """Adapter for the BrowserRequest to handle status messages.
    
    Let's make sure that this implementation actually fulfills the
    'IStatusMessage' API.

      >>> from zope.interface.verify import verifyClass
      >>> verifyClass(IStatusMessage, StatusMessage)
      True
    """
    implements(IStatusMessage)

    def __init__(self, context):
        self.context = context # the context must be the request

    def addStatusMessage(self, text, type=''):
        """Add a status message.
        """
        text = translate(text, context=self.context)
        annotations = IAnnotations(self.context)

        old = annotations.get(STATUSMESSAGEKEY, self.context.cookies.get(STATUSMESSAGEKEY))
        value = _encodeCookieValue(text, type, old=old)
        self.context.RESPONSE.setCookie(STATUSMESSAGEKEY, value, path='/')
        annotations[STATUSMESSAGEKEY] = value

    def showStatusMessages(self):
        """Removes all status messages and returns them for display.
        """
        annotations = IAnnotations(self.context)
        value = annotations.get(STATUSMESSAGEKEY, self.context.cookies.get(STATUSMESSAGEKEY))
        if value is None:
            return []
        value = _decodeCookieValue(value)
        # clear the existing cookie entries
        self.context.cookies[STATUSMESSAGEKEY] = None
        self.context.RESPONSE.expireCookie(STATUSMESSAGEKEY, path='/')
        annotations[STATUSMESSAGEKEY] = None
        return value


def _encodeCookieValue(text, type, old=None):
    """Encodes text and type to a list of Messages. If there is already some old
       existing list, add the new Message at the end but don't add duplicate
       messages.
    """
    results = []
    message = Message(text, type=type)

    if old is not None:
        results = _decodeCookieValue(old)
    if not message in results:
        results.append(message)

    messages = ''.join([r.encode() for r in results])
    return encodestring(messages).rstrip()

def _decodeCookieValue(string):
    """Decode a cookie value to a list of Messages.
    """
    results = []
    # Return nothing if the cookie is marked as deleted
    if string == 'deleted':
        return results
    # Try to decode the cookie value
    try:
        value = decodestring(string)
        while len(value) > 1: # at least 2 bytes of data
            message, value = decode(value)
            if message is not None:
                results.append(message)
    except (binascii.Error, UnicodeEncodeError):
        logger.log(logging.ERROR, '%s \n%s',
                   'Unexpected value in statusmessages cookie',
                   sys.exc_value
                   )
        return []

    return results



# -*- coding: UTF-8 -*-
"""
    StatusMessage adapter tests.
"""

import unittest

def test_directives():
    """
    Test status messages

    First some boilerplate.

      >>> from zope.component.testing import setUp
      >>> setUp()

      >>> import Products.Five
      >>> import Products.statusmessages

      >>> from Products.Five import zcml
      >>> zcml.load_config('meta.zcml', Products.Five)
      >>> zcml.load_config('configure.zcml', Products.statusmessages)

    Now lets make sure we can actually adapt the request.

      >>> from Products.statusmessages.interfaces import IStatusMessage
      >>> status = IStatusMessage(self.app.REQUEST)
      >>> IStatusMessage.providedBy(status)
      True

    We also need the request to be annotatable:

      >>> from zope.interface import directlyProvides
      >>> from zope.annotation.interfaces import IAttributeAnnotatable
      >>> directlyProvides(self.app.REQUEST, IAttributeAnnotatable)

    The dummy request we have is a bit limited, so we need a simple method
    to fake a real request/response for the cookie handling. Basically it
    puts all entries from RESPONSE.cookies into REQUEST.cookies but shifts
    the real values into the right place as browsers would do it.
      
      >>> def fakePublish(request):
      ...     cookies = request.RESPONSE.cookies.copy()
      ...     new_cookies = {}
      ...     for key in cookies.keys():
      ...         new_cookies[key] = cookies[key]['value']
      ...     request.cookies = new_cookies
      ...     request.RESPONSE.cookies = {}

      >>> request = self.app.REQUEST
      >>> status = IStatusMessage(request)

    Make sure there's no stored message.

      >>> len(status.showStatusMessages())
      0

    Add one message
      
      >>> status.addStatusMessage(u'test', type=u'info')

    Now check the results

      >>> messages = status.showStatusMessages()
      >>> len(messages)
      1

      >>> messages[0].message
      u'test'

      >>> messages[0].type
      u'info'

    Make sure messages are removed

      >>> len(status.showStatusMessages())
      0

    Since we accessed the message prior to publishing the page, we must 
    ensure that the messages have been removed from the cookies

      >>> fakePublish(request)
      >>> len(status.showStatusMessages())
      0

    Now we repeat the test, only this time we publish the page prior to
    retrieving the messages

    Add one message
      
      >>> status.addStatusMessage(u'test', type=u'info')

    Publish the request

      >>> fakePublish(request)

    Now check the results

      >>> messages = status.showStatusMessages()
      >>> len(messages)
      1

      >>> messages[0].message
      u'test'

      >>> messages[0].type
      u'info'

    Make sure messages are removed

      >>> len(status.showStatusMessages())
      0

    Add two messages (without publishing)

      >>> status.addStatusMessage(u'test', type=u'info')
      >>> status.addStatusMessage(u'test1', u'warn')
      
    And check the results again

      >>> messages = status.showStatusMessages()
      >>> len(messages)
      2

      >>> test = messages[1]

      >>> test.message
      u'test1'

      >>> test.type
      u'warn'

    Make sure messages are removed again

      >>> len(status.showStatusMessages())
      0

    Add two messages (with publishing)

      >>> status.addStatusMessage(u'test', type=u'info')
      >>> fakePublish(request)
      >>> status.addStatusMessage(u'test1', u'warn')
      
    And check the results again

      >>> fakePublish(request)
      >>> messages = status.showStatusMessages()
      >>> len(messages)
      2

      >>> test = messages[1]

      >>> test.message
      u'test1'

      >>> test.type
      u'warn'

    Make sure messages are removed again

      >>> len(status.showStatusMessages())
      0

    Add two identical messages

      >>> status.addStatusMessage(u'test', type=u'info')
      >>> status.addStatusMessage(u'test', type=u'info')

    And check the results again

      >>> fakePublish(request)
      >>> messages = status.showStatusMessages()
      >>> len(messages)
      1

      >>> test = messages[0]

      >>> test.message
      u'test'

      >>> test.type
      u'info'

    Make sure messages are removed again

      >>> len(status.showStatusMessages())
      0

    Test incredibly long messages:

      >>> status.addStatusMessage(u'm' * 0x400, type=u't' * 0x20)

      And check the results again

      >>> fakePublish(request)
      >>> messages = status.showStatusMessages()
      >>> len(messages)
      1

      >>> test = messages[0]

      >>> test.message == u'm' * 0x3FF
      True

      >>> test.type == u't' * 0x1F
      True

      >>> from zope.component.testing import tearDown
      >>> tearDown()
    """

def test_suite():
    from Testing.ZopeTestCase import ZopeDocTestSuite
    return ZopeDocTestSuite()

if __name__ == '__main__':
    unittest.main(defaultTest="test_suite")



from setuptools import setup, find_packages
import sys, os

version = '3.0.2'

setup(name='Products.statusmessages',
      version=version,
      description="statusmessages provides an easy way of handling "
                  "internationalized status messages managed via an "
                  "BrowserRequest adapter storing status messages in "
                  "client-side cookies. It requires Zope >= 2.10.",
      long_description="""\
      """,
      classifiers=[
        "Framework :: Plone",
        "Framework :: Zope2",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python",
      ],
      keywords='Zope CMF Plone status messages i18n',
      author='Hanno Schlichting',
      author_email='plone-developers@lists.sourceforge.net',
      url='http://svn.plone.org/svn/collective/statusmessages/trunk',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['Products'],
      include_package_data=True,
      zip_safe=False,
      download_url='http://plone.org/products/statusmessages/releases',
      install_requires=[
        'setuptools',
      ],
)



"""
    Encoding tests.
"""

import unittest

def test_encoding():
    r"""
    Test message encoding:

      >>> from Products.statusmessages.message import Message
      >>> from Products.statusmessages.message import decode

      >>> m = Message(u'spam', u'eggs')
      >>> m.encode()
      '\x00\x84spameggs'

      >>> decode(m.encode())[0] == m
      True

      >>> m = Message(u'spam')
      >>> m.encode()
      '\x00\x80spam'

      >>> decode(m.encode())[0] == m
      True
    """

def test_decoding():
    r"""
    Test message decoding:

      >>> from Products.statusmessages.message import Message
      >>> from Products.statusmessages.message import decode

    Craft a wrong value:

      >>> m, rem = decode('\x01\x84spameggs')
      >>> m.message, m.type
      (u'spameggs', u'')

      >>> rem
      ''

    Craft another wrong value:

      >>> m, rem = decode('\x00\x24spameggs')
      >>> m.message, m.type
      (u's', u'pame')

      >>> rem
      'ggs'

    And another wrong value:

      >>> m, rem = decode('\x00spameggs')
      >>> m.message, m.type
      (u'pam', u'eggs')

      >>> rem
      ''

    And yet another wrong value:

      >>> m, rem = decode('')
      >>> m is None, rem is ''
      (True, True)
    """


def test_suite():
    from Testing.ZopeTestCase import ZopeDocTestSuite
    return ZopeDocTestSuite()

if __name__ == '__main__':
    unittest.main(defaultTest="test_suite")
